package com.qmth.distributed.print.business.bean.dto.report;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.annotations.ApiModelProperty;
import org.jetbrains.annotations.NotNull;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

/**
 * @Description: 课程目标值dto
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/2/26
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
public class CourseTargetValueDto implements Serializable, Comparable<CourseTargetValueDto> {

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "课程目标id")
    private Long targetId;

    @ApiModelProperty(value = "课程目标名称")
    private String targetName;

    @ApiModelProperty(value = "毕业要求")
    private String graduationRequirement;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "评价方式id")
    private Long evaluationId;

    @ApiModelProperty(value = "评价方式")
    private String evaluation;

    @ApiModelProperty(value = "评价方式描述")
    private String evaluationDesc;

    @ApiModelProperty(value = "知识点集合")
    private List<DimensionDto> dimensionList;

    @ApiModelProperty(value = "目标整体权重")
    private BigDecimal targetWeight;

    @ApiModelProperty(value = "平时成绩信息")
    private UsualScoreDto usualScoreDto;

    @ApiModelProperty(value = "期末成绩权重")
    private FinalScoreDto finalScoreDto;

    public CourseTargetValueDto() {

    }

    public CourseTargetValueDto(Long targetId, String targetName) {
        this.targetId = targetId;
        this.targetName = targetName;
    }

    public CourseTargetValueDto(CourseTargetDto courseTargetDto) {
        this.targetId = courseTargetDto.getTargetId();
        this.targetName = courseTargetDto.getTargetName();
        this.graduationRequirement = courseTargetDto.getGraduationRequirement();
        setValue(courseTargetDto);
    }

    public void setValue(CourseTargetDto courseTargetDto) {
        this.evaluationId = courseTargetDto.getEvaluationId();
        this.evaluation = courseTargetDto.getEvaluation();
        this.evaluationDesc = courseTargetDto.getEvaluationDesc();
    }

    public void updateInfo(CourseTargetDto courseTargetDto) {
        setValue(courseTargetDto);
    }

    public String getGraduationRequirement() {
        return graduationRequirement;
    }

    public void setGraduationRequirement(String graduationRequirement) {
        this.graduationRequirement = graduationRequirement;
    }

    public Long getEvaluationId() {
        return evaluationId;
    }

    public void setEvaluationId(Long evaluationId) {
        this.evaluationId = evaluationId;
    }

    public String getEvaluation() {
        return evaluation;
    }

    public void setEvaluation(String evaluation) {
        this.evaluation = evaluation;
    }

    public String getEvaluationDesc() {
        return evaluationDesc;
    }

    public void setEvaluationDesc(String evaluationDesc) {
        this.evaluationDesc = evaluationDesc;
    }

    public List<DimensionDto> getDimensionList() {
        return dimensionList;
    }

    public void setDimensionList(List<DimensionDto> dimensionList) {
        this.dimensionList = dimensionList;
    }

    public Long getTargetId() {
        return targetId;
    }

    public void setTargetId(Long targetId) {
        this.targetId = targetId;
    }

    public String getTargetName() {
        return targetName;
    }

    public void setTargetName(String targetName) {
        this.targetName = targetName;
    }

    public BigDecimal getTargetWeight() {
        return targetWeight;
    }

    public void setTargetWeight(BigDecimal targetWeight) {
        this.targetWeight = targetWeight;
    }

    public UsualScoreDto getUsualScoreDto() {
        return usualScoreDto;
    }

    public void setUsualScoreDto(UsualScoreDto usualScoreDto) {
        this.usualScoreDto = usualScoreDto;
    }

    public FinalScoreDto getFinalScoreDto() {
        return finalScoreDto;
    }

    public void setFinalScoreDto(FinalScoreDto finalScoreDto) {
        this.finalScoreDto = finalScoreDto;
    }

    @Override
    public int compareTo(@NotNull CourseTargetValueDto o) {
        if (o.getTargetId().longValue() < this.getTargetId().longValue()) {
            return 1;
        } else if (o.getTargetId().longValue() > this.getTargetId().longValue()) {
            return -1;
        } else {
            return 0;
        }
    }
}
