package com.qmth.distributed.print.api;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.data.*;
import com.deepoove.poi.data.style.CellStyle;
import com.deepoove.poi.data.style.ParagraphStyle;
import com.deepoove.poi.data.style.RowStyle;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.dto.report.*;
import com.qmth.distributed.print.business.bean.result.ScoreResult;
import com.qmth.distributed.print.business.bean.result.report.ReportResult;
import com.qmth.distributed.print.business.bean.result.report.word.*;
import com.qmth.distributed.print.business.entity.TRBasicInfo;
import com.qmth.distributed.print.business.entity.TRExamStudent;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.distributed.print.business.service.TCFinalScoreService;
import com.qmth.distributed.print.business.service.TRBasicInfoService;
import com.qmth.distributed.print.business.service.TRExamStudentService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import io.swagger.annotations.*;
import org.apache.poi.xwpf.usermodel.ParagraphAlignment;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 报告基本情况表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-报告基本情况Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
public class TRBasicInfoController {

    @Resource
    PrintCommonService printCommonService;

    @Resource
    MarkPaperService markPaperService;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    TRExamStudentService trExamStudentService;

    @ApiOperation(value = "报告管理列表")
    @RequestMapping(value = "/report/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = ScoreResult.class)})
    public Object reportList(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                             @ApiParam(value = "课程编码") @RequestParam(required = false) String courseCode,
                             @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                             @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(printCommonService.scoreList(new Page<>(pageNumber, pageSize), examId, courseCode));
    }

    @ApiOperation(value = "查看报告")
    @RequestMapping(value = "/report/view", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "查看报告", response = ReportResult.class)})
    @Transactional
    public Object reportView(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        Objects.requireNonNull(markPaper, "未找到科目信息");

        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TRBasicInfo trBasicInfo = trBasicInfoService.getOne(new QueryWrapper<TRBasicInfo>().lambda().eq(TRBasicInfo::getExamId, examId).eq(TRBasicInfo::getCourseCode, courseCode).eq(TRBasicInfo::getPaperNumber, paperNumber));

        if (Objects.isNull(trBasicInfo)) {
            trBasicInfo = trBasicInfoService.getReportView(trBasicInfo, markPaper, sysUser.getId());
        } else {
            ReportCourseBasicInfoDto reportCourseBasicInfoDto = new ReportCourseBasicInfoDto(trBasicInfo);
            ReportCourseEvaluationSpreadDto reportCourseEvaluationSpreadDto = Objects.nonNull(trBasicInfo.getCourseEvaluationSpread()) ? JSONObject.parseObject(trBasicInfo.getCourseEvaluationSpread(), ReportCourseEvaluationSpreadDto.class) : null;
            ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = Objects.nonNull(trBasicInfo.getCourseEvaluationResult()) ? JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class) : null;
            ReportCourseEvaluationResultDetailDto reportCourseEvaluationResultDetailDto = null;
            List<TRExamStudent> trExamStudentList = trExamStudentService.list(new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()));
            if (!CollectionUtils.isEmpty(trExamStudentList)) {
                List<ReportExamStudentDto> examStudentList = new ArrayList<>(trExamStudentList.size());
                for (TRExamStudent trExamStudent : trExamStudentList) {
                    examStudentList.add(new ReportExamStudentDto(trExamStudent));
                }
                reportCourseEvaluationResultDetailDto = new ReportCourseEvaluationResultDetailDto(examStudentList);
            }
            trBasicInfo.setReportResult(new ReportResult(new ReportCommonDto(examId, courseCode, markPaper.getCourseName(), paperNumber), reportCourseBasicInfoDto, reportCourseEvaluationSpreadDto, reportCourseEvaluationResultDto, reportCourseEvaluationResultDetailDto));
        }

        trBasicInfo.updateInfo(sysUser.getId());
        //课程目标达成评价明细结果-课程目标达成评价值图
        trBasicInfoService.saveOrUpdate(trBasicInfo);
        return ResultUtil.ok(trBasicInfo.getReportResult());
    }

    @ApiOperation(value = "保存报告")
    @RequestMapping(value = "/report/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存报告", response = Object.class)})
    @Transactional
    public Object reportSave(@ApiParam(value = "保存报告结构", required = true) @Valid @RequestBody TRBasicInfo trBasicInfo, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TRBasicInfo trBasicInfoDb = trBasicInfoService.getOne(new QueryWrapper<TRBasicInfo>().lambda().eq(TRBasicInfo::getExamId, trBasicInfo.getExamId()).eq(TRBasicInfo::getCourseCode, trBasicInfo.getCourseCode()).eq(TRBasicInfo::getPaperNumber, trBasicInfo.getPaperNumber()));
        if (Objects.isNull(trBasicInfoDb)) {
            trBasicInfoDb = new TRBasicInfo(trBasicInfo, sysUser.getId());
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(trBasicInfo.getExamId(), trBasicInfo.getPaperNumber());
            Objects.requireNonNull(markPaper, "未找到科目信息");

            trBasicInfoDb = trBasicInfoService.getReportView(trBasicInfoDb, markPaper, sysUser.getId());
        } else {
            trBasicInfoDb.updateInfo(trBasicInfo, sysUser.getId());
        }
        return ResultUtil.ok(trBasicInfoService.saveOrUpdate(trBasicInfoDb));
    }

    @ApiOperation(value = "导出报告")
    @RequestMapping(value = "/report/export", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EXPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void reportExport(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) throws IOException {
        TRBasicInfo trBasicInfo = trBasicInfoService.getOne(new QueryWrapper<TRBasicInfo>().lambda().eq(TRBasicInfo::getExamId, examId).eq(TRBasicInfo::getCourseCode, courseCode).eq(TRBasicInfo::getPaperNumber, paperNumber));
        Objects.requireNonNull(trBasicInfo, "没有报告信息");
        Objects.requireNonNull(trBasicInfo.getCourseEvaluationResultDetail(), "没有课程目标信息");

        String period = Objects.nonNull(trBasicInfo.getPeriod()) ? trBasicInfo.getPeriod() : null;
        String credit = Objects.nonNull(trBasicInfo.getCredit()) ? trBasicInfo.getCredit() : null;
        if (Objects.nonNull(credit)) {
            period = period + "/" + credit;
        }
        ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = Objects.nonNull(trBasicInfo.getCourseEvaluationResult()) ? JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class) : null;
        CourseBasicBean courseBasicBean = new CourseBasicBean(trBasicInfo.getCourseName(),
                trBasicInfo.getCourseCode(), null, trBasicInfo.getCourseType(), period, null, null, trBasicInfo.getDirector(), DateUtil.format(new Date(trBasicInfo.getCreateTime()), SystemConstant.DEFAULT_DATE_PATTERN), "测试：我们的目标是没有目标", Objects.nonNull(reportCourseEvaluationResultDto) && !CollectionUtils.isEmpty(reportCourseEvaluationResultDto.getTargetList()) ? reportCourseEvaluationResultDto.getTargetList().size() : null);

        CourseReportBean courseReportBean = new CourseReportBean(trBasicInfo.getOpenTime() + "《测试输出报告》", courseBasicBean);

        //targetMap
        List<CourseTargetValueDto> courseTargetValueDtoList = JSONArray.parseArray(trBasicInfo.getCourseEvaluationResultDetail(), CourseTargetValueDto.class);
//        Map<Long, CourseTargetValueDto> courseTargetValueDtoMap = courseTargetValueDtoList.stream().collect(Collectors.toMap(CourseTargetValueDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));

        //table1课程目标目标与毕业要求指标点的对应关系
        RowRenderData header1 = Rows.of("课程目标", "毕业要求指标点").bgColor("F2F2F2").center().create();
        Tables.TableBuilder tableBuilder1 = Tables.ofPercentWidth("100%");
        tableBuilder1.addRow(header1);

        List<String> header2_1List = new ArrayList<>(Arrays.asList("课程目标", "支撑毕业要求")), header2_2List = new ArrayList<>();
        header2_2List.addAll(header2_1List);
        header2_1List.add("考核/评价环节及目标分值");

        for (int i = 0; i < courseTargetValueDtoList.size(); i++) {
            CourseTargetValueDto courseTargetValueDto = courseTargetValueDtoList.get(i);
//            Map<String, BigDecimal> usualScoreWeightMap = courseTargetValueDto.getUsualScoreWeight();
//            usualScoreWeightMap.forEach((k1, v1) -> {
//                header2_1List.add("");
//                header2_2List.add(k1);
//            });

            CourseTargetTableBean1 courseTargetTableBean1 = new CourseTargetTableBean1(courseTargetValueDto.getTargetName(), courseTargetValueDto.getGraduationRequirement());
            RowRenderData row = Rows.of(courseTargetTableBean1.getName(), courseTargetTableBean1.getTarget()).rowStyle(this.getRowStyle()).create();
            tableBuilder1.addRow(row);
        }
        header2_1List.add("目标分值统计");
        header2_2List.add("期末考试");
        header2_2List.add("目标分值统计");

        TableRenderData tableRenderData1 = tableBuilder1.left().create();
        courseReportBean.setCourseTargetTable1(tableRenderData1);

        //TODO 等待曹子轩接口table2-课程目标达成评价依据-课程目标毕业要求
        RowRenderData header2_1 = Rows.of(header2_1List.toArray(new String[header2_1List.size()])).bgColor("F2F2F2").center().create();
        RowRenderData header2_2 = Rows.of(header2_2List.toArray(new String[header2_1List.size()])).bgColor("F2F2F2").center().create();

        List<CourseTargetTableBean2> courseTargetTableBean2List = new ArrayList<>();
        AtomicReference<BigDecimal> bigDecimalExpressionSum = new AtomicReference<>(new BigDecimal(0));
        AtomicReference<BigDecimal> homeworkSum = new AtomicReference<>(new BigDecimal(0));
        AtomicReference<BigDecimal> testSum = new AtomicReference<>(new BigDecimal(0));
        AtomicReference<BigDecimal> examSum = new AtomicReference<>(new BigDecimal(0));
        AtomicReference<BigDecimal> targetSum2 = new AtomicReference<>(new BigDecimal(0));
        AtomicInteger atomicReference = new AtomicInteger(0);
        for (int i = 0; i < courseTargetValueDtoList.size(); i++) {
            CourseTargetValueDto courseTargetValueDto = courseTargetValueDtoList.get(i);
            CourseTargetTableBean2 courseTargetTableBean2 = new CourseTargetTableBean2(courseTargetValueDto.getTargetName(), courseTargetValueDto.getGraduationRequirement(), new BigDecimal(atomicReference.get() + 2), new BigDecimal(atomicReference.get() + 3), new BigDecimal(atomicReference.get() + 4), new BigDecimal(atomicReference.get() + 5), new BigDecimal(atomicReference.get() + 10));
            bigDecimalExpressionSum.set(bigDecimalExpressionSum.get().add(courseTargetTableBean2.getExpression()));
            homeworkSum.set(homeworkSum.get().add(courseTargetTableBean2.getHomework()));
            testSum.set(testSum.get().add(courseTargetTableBean2.getTest()));
            examSum.set(examSum.get().add(courseTargetTableBean2.getExam()));
            targetSum2.set(targetSum2.get().add(courseTargetTableBean2.getTargetSum()));
            courseTargetTableBean2List.add(courseTargetTableBean2);
            atomicReference.decrementAndGet();
        }

        RowRenderData[] rowRenderData2 = new RowRenderData[courseTargetTableBean2List.size() + 3];
        for (int i = 0; i < courseTargetTableBean2List.size(); i++) {
            CourseTargetTableBean2 courseTargetTableBean2 = courseTargetTableBean2List.get(i);
            rowRenderData2[i + 2] = Rows.of(courseTargetTableBean2.getName(), courseTargetTableBean2.getTarget(), courseTargetTableBean2.getExpression().toString(), courseTargetTableBean2.getHomework().toString(), courseTargetTableBean2.getTest().toString(), courseTargetTableBean2.getExam().toString(), courseTargetTableBean2.getTargetSum().toString()).rowStyle(this.getRowStyle()).create();
//            rowRenderData[i + 2] = Rows.of(this.getCell(courseTargetTableBean2.getName()), this.getCell(courseTargetTableBean2.getTarget()), this.getCell(courseTargetTableBean2.getExpression().toString()), this.getCell(courseTargetTableBean2.getHomework().toString()), this.getCell(courseTargetTableBean2.getTest().toString()), this.getCell(courseTargetTableBean2.getExam().toString()), this.getCell(courseTargetTableBean2.getTargetSum().toString())).create();
        }
        rowRenderData2[0] = header2_1;
        rowRenderData2[1] = header2_2;

        RowRenderData header2_foot = Rows.of("合计", "", bigDecimalExpressionSum.toString(), homeworkSum.toString(), testSum.toString(), examSum.toString(), targetSum2.toString()).rowStyle(this.getRowStyle()).create();
        rowRenderData2[rowRenderData2.length - 1] = header2_foot;

        // 表格合并,根据坐标
        MergeCellRule mergeCellRule2 = MergeCellRule.builder().map(MergeCellRule.Grid.of(0, 0), MergeCellRule.Grid.of(1, 0)).
                map(MergeCellRule.Grid.of(0, 1), MergeCellRule.Grid.of(1, 1)).
                map(MergeCellRule.Grid.of(0, 2), MergeCellRule.Grid.of(0, 5)).
                map(MergeCellRule.Grid.of(0, 6), MergeCellRule.Grid.of(1, 6)).
                map(MergeCellRule.Grid.of(rowRenderData2.length - 1, 0), MergeCellRule.Grid.of(rowRenderData2.length - 1, 1)).
                build();

        Tables.TableBuilder tableBuilder2 = Tables.ofPercentWidth("100%");
        for (int i = 0; i < rowRenderData2.length; i++) {
            tableBuilder2.addRow(rowRenderData2[i]);
        }
        TableRenderData tableRenderData2 = tableBuilder2.mergeRule(mergeCellRule2).left().create();
//        TableRenderData tableRenderData2 = Tables.of(rowRenderData2).mergeRule(mergeCellRule2)
//                .width(17f, new double[]{17 / 6, 22 / 6, 15 / 6, 15 / 6, 15 / 6, 15 / 6, 17 / 6})
//                .left().create();
        courseReportBean.setCourseTargetTable2(tableRenderData2);

        //TODO 等待曹子轩接口table3-课程目标达成评价依据-平时作业目标分值
        RowRenderData header3_1 = Rows.of("序号", "项目", "考核内容及要求", "目标分值", "对应的课程目标").bgColor("F2F2F2").center().create();
        List<CourseTargetTableBean3> courseTargetTableBean3List = new ArrayList<>();
        BigDecimal targetSum3 = new BigDecimal(0);
        String tableFootTitle3 = "合计${sum}分，占课程总评成绩的30%";
        for (int i = 0; i < 2; i++) {
            CourseTargetTableBean3 courseTargetTableBean3 = new CourseTargetTableBean3((i + 1), "项目" + (i + 1), "内容" + (i + 1), new BigDecimal(i + 3).doubleValue(), "目标分值" + (i + 1));
            targetSum3 = targetSum3.add(new BigDecimal(courseTargetTableBean3.getTarget()));
            courseTargetTableBean3List.add(courseTargetTableBean3);
        }
        tableFootTitle3 = tableFootTitle3.replace("${sum}", targetSum3.toString());

        RowRenderData[] rowRenderData3 = new RowRenderData[courseTargetTableBean3List.size() + 2];
        for (int i = 0; i < courseTargetTableBean3List.size(); i++) {
            CourseTargetTableBean3 courseTargetTableBean3 = courseTargetTableBean3List.get(i);
            rowRenderData3[i + 1] = Rows.of(courseTargetTableBean3.getNum().toString(), courseTargetTableBean3.getProject(), courseTargetTableBean3.getContent(), courseTargetTableBean3.getTarget().toString(), courseTargetTableBean3.getCourseTarget()).rowStyle(this.getRowStyle()).create();
        }
        rowRenderData3[0] = header3_1;
        RowRenderData header3_foot = Rows.of(tableFootTitle3, "", "", "", "").center().create();
        rowRenderData3[rowRenderData3.length - 1] = header3_foot;

        // 表格合并,根据坐标
        MergeCellRule mergeCellRule3 = MergeCellRule.builder().map(MergeCellRule.Grid.of(rowRenderData3.length - 1, 0), MergeCellRule.Grid.of(rowRenderData3.length - 1, 4)).
                build();

//        TableRenderData tableRenderData3 = Tables.of(rowRenderData3).mergeRule(mergeCellRule3)
//                .width(17f, new double[]{10 / 6, 15 / 6, 24 / 6, 15 / 6, 20 / 6}).left().create();
        Tables.TableBuilder tableBuilder3 = Tables.ofPercentWidth("100%");
        for (int i = 0; i < rowRenderData3.length; i++) {
            tableBuilder3.addRow(rowRenderData3[i]);
        }
        TableRenderData tableRenderData3 = tableBuilder3.mergeRule(mergeCellRule3).left().create();
        courseReportBean.setCourseTargetTable3(tableRenderData3);

        //table4-课程目标达成评价依据-期末考试知识单元目标分值
        RowRenderData header4_1 = Rows.of("序号", "考核项目", "考核内容所涵盖的知识单元", "目标分值", "对应的课程目标").bgColor("F2F2F2").center().create();
        List<CourseTargetTableBean4> courseTargetTableBean4List = new ArrayList<>();
        BigDecimal sum4 = new BigDecimal(0);
        Integer seq4 = 0;
        for (int i = 0; i < courseTargetValueDtoList.size(); i++) {
            CourseTargetValueDto courseTargetValueDto = courseTargetValueDtoList.get(i);
            List<DimensionDto> dimensionDtoList = courseTargetValueDto.getDimensionList();
            for (DimensionDto dimensionDto : dimensionDtoList) {
                sum4 = sum4.add(new BigDecimal(dimensionDto.getDimensionScore()));
                seq4 = seq4 + 1;
                CourseTargetTableBean4 courseTargetTableBean4 = new CourseTargetTableBean4(seq4, courseTargetValueDto.getGraduationRequirement(), dimensionDto.getDimensionName(), dimensionDto.getDimensionScore(), courseTargetValueDto.getTargetName());
                courseTargetTableBean4List.add(courseTargetTableBean4);
            }
        }
        String tableFootTitle4 = "合计${sum}分，占课程总评成绩的70%";
        tableFootTitle4 = tableFootTitle4.replace("${sum}", sum4.toString());

        RowRenderData[] rowRenderData4 = new RowRenderData[courseTargetTableBean4List.size() + 2];
        for (int i = 0; i < courseTargetTableBean4List.size(); i++) {
            CourseTargetTableBean4 courseTargetTableBean4 = courseTargetTableBean4List.get(i);
            rowRenderData4[i + 1] = Rows.of(courseTargetTableBean4.getNum().toString(), courseTargetTableBean4.getProject(), courseTargetTableBean4.getContent(), courseTargetTableBean4.getTarget().toString(), courseTargetTableBean4.getCourseTarget()).rowStyle(this.getRowStyle()).create();
        }
        rowRenderData4[0] = header4_1;
        RowRenderData header4_foot = Rows.of(tableFootTitle4, "", "", "", "").center().create();
        rowRenderData4[rowRenderData4.length - 1] = header4_foot;

        // 表格合并,根据坐标
        MergeCellRule mergeCellRule4 = MergeCellRule.builder().map(MergeCellRule.Grid.of(rowRenderData4.length - 1, 0), MergeCellRule.Grid.of(rowRenderData4.length - 1, 4)).
                build();

//        TableRenderData tableRenderData4 = Tables.of(rowRenderData3).mergeRule(mergeCellRule4)
//                .width(17f, new double[]{10 / 6, 15 / 6, 24 / 6, 15 / 6, 20 / 6})
//                .left().create();
        Tables.TableBuilder tableBuilder4 = Tables.ofPercentWidth("100%");
        for (int i = 0; i < rowRenderData4.length; i++) {
            tableBuilder4.addRow(rowRenderData4[i]);
        }
        TableRenderData tableRenderData4 = tableBuilder4.mergeRule(mergeCellRule4).left().create();
        courseReportBean.setCourseTargetTable4(tableRenderData4);

        //examstudent-课程目标达成评价依据-考生
        List<TRExamStudent> trExamStudentList = trExamStudentService.list(new QueryWrapper<TRExamStudent>().lambda().eq(TRExamStudent::getrBasicInfoId, trBasicInfo.getId()));
        if (!CollectionUtils.isEmpty(trExamStudentList)) {
//            List<FinalScoreResult> finalScoreResultList = tcFinalScoreService.examStudentOverview(trBasicInfo.getExamId(), trBasicInfo.getCourseCode(), trBasicInfo.getPaperNumber());//考生成绩
//            Map<String, FinalScoreResult> finalScoreResultMap = finalScoreResultList.stream().collect(Collectors.toMap(FinalScoreResult::getStudentCode, Function.identity(), (dto1, dto2) -> dto1));

            List<CellRenderData> examStudent_cells_1 = new ArrayList<>(), examStudent_cells_2 = new ArrayList<>(), examStudent_cells_3 = new ArrayList<>(), examStudent_cells_4 = new ArrayList<>(), examStudent_cells_5 = new ArrayList<>();
            examStudent_cells_1.add(Cells.of("序号").center().create());
            examStudent_cells_1.add(Cells.of("学号").center().create());
            examStudent_cells_1.add(Cells.of("姓名").center().create());
            examStudent_cells_1.add(Cells.of("行政班级").center().create());
            examStudent_cells_2.addAll(examStudent_cells_1);

            examStudent_cells_3.add(Cells.of("课程目标考核要素的目标分").center().create());
            examStudent_cells_3.add(Cells.of("").center().create());
            examStudent_cells_3.add(Cells.of("").center().create());
            examStudent_cells_3.add(Cells.of("").center().create());

            examStudent_cells_4.add(Cells.of("课程目标考核要素的平均分").center().create());
            examStudent_cells_4.add(Cells.of("").center().create());
            examStudent_cells_4.add(Cells.of("").center().create());
            examStudent_cells_4.add(Cells.of("").center().create());

            examStudent_cells_5.add(Cells.of("各课程目标平均分").center().create());
            examStudent_cells_5.add(Cells.of("").center().create());
            examStudent_cells_5.add(Cells.of("").center().create());
            examStudent_cells_5.add(Cells.of("").center().create());

            List<ExamStudentTableBean> examStudentTableBeanList = new ArrayList<>();
            int seq5 = 0;
            Map<Long, Integer> targetUsualScoreSizeMap = new HashMap<>();
            trExamStudentList.remove(trExamStudentList.size() - 1);
            TRExamStudent trExamStudentTemp = trExamStudentList.get(0);

            //首行
            if (Objects.nonNull(trExamStudentTemp.getResultDetail())) {
                List<ReportExamStudentTargetDto> reportExamStudentTargetDtoList = JSONArray.parseArray(trExamStudentTemp.getResultDetail(), ReportExamStudentTargetDto.class);
                for (ReportExamStudentTargetDto reportExamStudentTargetDto : reportExamStudentTargetDtoList) {
                    examStudent_cells_1.add(Cells.of(reportExamStudentTargetDto.getTargetName()).center().create());
                    ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = reportExamStudentTargetDto.getUsualScore();
                    //平常作业
                    if (Objects.nonNull(reportExamStudentUsualScoreDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreDto.getScoreList())) {
                        List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                        for (ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto : reportExamStudentUsualScoreObjDtoList) {
                            if (!targetUsualScoreSizeMap.containsKey(reportExamStudentTargetDto.getTargetId())) {
                                targetUsualScoreSizeMap.put(reportExamStudentTargetDto.getTargetId(), 1);
                            } else {
                                Integer size = targetUsualScoreSizeMap.get(reportExamStudentTargetDto.getTargetId());
                                targetUsualScoreSizeMap.put(reportExamStudentTargetDto.getTargetId(), size++);
                            }
                            examStudent_cells_1.add(Cells.of("").center().create());
                            examStudent_cells_2.add(Cells.of(reportExamStudentUsualScoreObjDto.getName()).center().create());
                        }
                    }
                    //期末考试
                    ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = reportExamStudentTargetDto.getFinalScore();
                    if (Objects.nonNull(reportExamStudentFinalScoreDto) && !CollectionUtils.isEmpty(reportExamStudentFinalScoreDto.getDimensionList())) {
                        examStudent_cells_2.add(Cells.of("期末考试").center().create());
                    }
                }
                examStudent_cells_1.add(Cells.of("综合成绩").center().create());
                examStudent_cells_2.add(Cells.of("综合成绩").center().create());
            }

            for (int i = 0; i < trExamStudentList.size(); i++) {
                TRExamStudent trExamStudent = trExamStudentList.get(i);
                LinkedMultiValueMap<Long, Map<String, ExamStudentScoreBean>> scoreMap = new LinkedMultiValueMap<>();
                if (Objects.nonNull(trExamStudent.getResultDetail())) {
                    List<ReportExamStudentTargetDto> reportExamStudentTargetDtoList = JSONArray.parseArray(trExamStudent.getResultDetail(), ReportExamStudentTargetDto.class);
                    for (ReportExamStudentTargetDto reportExamStudentTargetDto : reportExamStudentTargetDtoList) {
                        if (Objects.nonNull(reportExamStudentTargetDto)) {
                            ReportExamStudentUsualScoreDto reportExamStudentUsualScoreDto = reportExamStudentTargetDto.getUsualScore();
                            //平常作业
                            if (Objects.nonNull(reportExamStudentUsualScoreDto) && !CollectionUtils.isEmpty(reportExamStudentUsualScoreDto.getScoreList())) {
                                List<ReportExamStudentUsualScoreObjDto> reportExamStudentUsualScoreObjDtoList = reportExamStudentUsualScoreDto.getScoreList();
                                for (ReportExamStudentUsualScoreObjDto reportExamStudentUsualScoreObjDto : reportExamStudentUsualScoreObjDtoList) {
                                    scoreMap.add(reportExamStudentTargetDto.getTargetId(), Collections.singletonMap(reportExamStudentUsualScoreObjDto.getName(), new ExamStudentScoreBean(reportExamStudentUsualScoreObjDto.getScore())));
                                }
                            }
                            //期末考试
                            ReportExamStudentFinalScoreDto reportExamStudentFinalScoreDto = reportExamStudentTargetDto.getFinalScore();
                            if (Objects.nonNull(reportExamStudentFinalScoreDto) && !CollectionUtils.isEmpty(reportExamStudentFinalScoreDto.getDimensionList())) {
                                Double sumScore = reportExamStudentFinalScoreDto.getDimensionList().stream().mapToDouble(DimensionDto::getDimensionScore).sum();
                                scoreMap.add(reportExamStudentTargetDto.getTargetId(), Collections.singletonMap("期末考试", new ExamStudentScoreBean(sumScore)));
                            }
                        }
                    }
                    seq5 = seq5 + 1;
                    examStudentTableBeanList.add(new ExamStudentTableBean(seq5, trExamStudent.getStudentCode(), trExamStudent.getName(), trExamStudent.getAdministrativeClass(), scoreMap, 0.0d));
                }
            }

            String title = trBasicInfo.getTeachingObject();
            String sumTitle = "学生共${sum}名";
            sumTitle = sumTitle.replace("${sum}", trExamStudentList.size() + "");

            courseReportBean.setExamStudentTitle1(title + sumTitle);
            courseReportBean.setExamStudentTitle2(title + "《" + courseReportBean.getCourseBasicBean().getCourseName() + "》");

            RowRenderData[] rowRenderDataExamStudent = new RowRenderData[trExamStudentList.size() + 5];

            RowRenderData examStudent_header_1 = new RowRenderData();
            examStudent_header_1.setCells(examStudent_cells_1);
            examStudent_header_1.setRowStyle(this.getHeadRowStyle());

            RowRenderData examStudent_header_2 = new RowRenderData();
            examStudent_header_2.setCells(examStudent_cells_2);
            examStudent_header_2.setRowStyle(this.getHeadRowStyle());

            for (int i = 0; i < examStudentTableBeanList.size(); i++) {
                ExamStudentTableBean e = examStudentTableBeanList.get(i);
                RowRenderData examStudent_row = new RowRenderData();
                List<CellRenderData> examStudent_cells = new ArrayList<>();
                examStudent_cells.add(Cells.of(e.getNum().toString()).create());
                examStudent_cells.add(Cells.of(e.getStudentCode()).create());
                examStudent_cells.add(Cells.of(e.getName()).create());
                examStudent_cells.add(Cells.of(e.getAdministrativeClass()).create());

                String key = null;
                LinkedMultiValueMap<Long, Map<String, ExamStudentScoreBean>> map = e.getMap();
                for (Map.Entry<Long, List<Map<String, ExamStudentScoreBean>>> entry : map.entrySet()) {
                    List<Map<String, ExamStudentScoreBean>> list = entry.getValue();
                    BigDecimal courseSumAvg = new BigDecimal(0);
                    for (int y = 0; y < list.size(); y++) {
                        Map<String, ExamStudentScoreBean> m = list.get(y);
                        for (Map.Entry<String, ExamStudentScoreBean> entry1 : m.entrySet()) {
                            courseSumAvg = courseSumAvg.add(new BigDecimal(entry1.getValue().getScore()));
                            entry1.getValue().setAllScore(new BigDecimal(new Random().nextInt(10) + 1).doubleValue());
                            key = entry1.getKey();
                            examStudent_cells.add(Cells.of(entry1.getValue().getScore().toString()).create());
                        }
                        if (y == list.size() - 1) {
                            courseSumAvg = courseSumAvg.divide(new BigDecimal(list.size()), 2, BigDecimal.ROUND_HALF_UP)
                                    .setScale(2, BigDecimal.ROUND_HALF_UP);
                            list.get(y).get(key).setAvgAllScore(courseSumAvg.doubleValue());
                        }
                    }
                }
                if (i == examStudentTableBeanList.size() - 1) {
                    LinkedMultiValueMap<Long, Map<String, ExamStudentScoreBean>> map1 = e.getMap();
                    for (Map.Entry<Long, List<Map<String, ExamStudentScoreBean>>> entry1 : map1.entrySet()) {
                        List<Map<String, ExamStudentScoreBean>> list1 = entry1.getValue();
                        for (int k = 0; k < list1.size(); k++) {
                            Map<String, ExamStudentScoreBean> m = list1.get(k);
                            for (Map.Entry<String, ExamStudentScoreBean> entry2 : m.entrySet()) {
                                examStudent_cells_3.add(Cells.of(entry2.getValue().getTargetAllAvgScore().toString()).create());
                                examStudent_cells_4.add(Cells.of(entry2.getValue().getAllScore().toString()).create());
                            }
                        }
                        examStudent_cells_5.add(Cells.of(list1.get(list1.size() - 1).get(key).getAvgAllScore().toString()).create());
                        for (int k = 1; k < list1.size(); k++) {
                            examStudent_cells_5.add(Cells.of("").create());
                        }
                    }
                    examStudent_cells_3.add(Cells.of(e.getSumScore().toString()).create());
                    examStudent_cells_4.add(Cells.of(e.getSumScore().toString()).create());
                    examStudent_cells_5.add(Cells.of("").create());
                }
                examStudent_cells.add(Cells.of(e.getSumScore().toString()).create());
                examStudent_row.setCells(examStudent_cells);
                examStudent_row.setRowStyle(this.getRowStyle());
                rowRenderDataExamStudent[i + 2] = examStudent_row;
            }

            rowRenderDataExamStudent[0] = examStudent_header_1;
            rowRenderDataExamStudent[1] = examStudent_header_2;
            RowRenderData examStudent_row3 = new RowRenderData();
            examStudent_row3.setCells(examStudent_cells_3);
            examStudent_row3.setRowStyle(this.getRowStyle());
            RowRenderData examStudent_row4 = new RowRenderData();
            examStudent_row4.setCells(examStudent_cells_4);
            examStudent_row4.setRowStyle(this.getRowStyle());
            RowRenderData examStudent_row5 = new RowRenderData();
            examStudent_row5.setCells(examStudent_cells_5);
            examStudent_row5.setRowStyle(this.getRowStyle());

            rowRenderDataExamStudent[rowRenderDataExamStudent.length - 3] = examStudent_row3;
            rowRenderDataExamStudent[rowRenderDataExamStudent.length - 2] = examStudent_row4;
            rowRenderDataExamStudent[rowRenderDataExamStudent.length - 1] = examStudent_row5;

            // 表格合并,根据坐标
            MergeCellRule mergeCellRuleExamStudent = MergeCellRule.builder().build();
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 0), MergeCellRule.Grid.of(1, 0)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 1), MergeCellRule.Grid.of(1, 1)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 2), MergeCellRule.Grid.of(1, 2)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, 3), MergeCellRule.Grid.of(1, 3)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 3, 0), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 3, 3)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 2, 0), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 2, 3)));
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, 0), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, 3)));

            //动态扩展列
            AtomicInteger initNum = new AtomicInteger(4);
            targetUsualScoreSizeMap.forEach((k, v) -> {
                int increaseNum = 1;
                while (increaseNum <= v) {
                    mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, initNum.get()), MergeCellRule.Grid.of(0, initNum.get() + increaseNum)));
                    mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, initNum.get()), MergeCellRule.Grid.of(rowRenderDataExamStudent.length - 1, initNum.get() + increaseNum)));
                    increaseNum++;
                }
                initNum.set(initNum.get() + increaseNum);
            });
            mergeCellRuleExamStudent.getMapping().add(new MergeCellRule.GridRule(MergeCellRule.Grid.of(0, initNum.get()), MergeCellRule.Grid.of(1, initNum.get())));

            Tables.TableBuilder tableBuilderExamStudent = Tables.ofPercentWidth("100%");
            for (int i = 0; i < rowRenderDataExamStudent.length; i++) {
                tableBuilderExamStudent.addRow(rowRenderDataExamStudent[i]);
            }
            TableRenderData tableRenderDataExamStudent = tableBuilderExamStudent.mergeRule(mergeCellRuleExamStudent).left().create();
            courseReportBean.setExamStudentTable1(tableRenderDataExamStudent);
        }

        XWPFTemplate template = XWPFTemplate.compile("/Users/king/git/teachcloud-server/distributed-print/src/main/resources/static/course_degree_report.docx").render(courseReportBean);
        template.writeToFile("/Users/king/Downloads/demo_course_degree_report.docx");
    }

    /**
     * 获取行样式
     *
     * @return
     */
    protected RowStyle getRowStyle() {
        RowStyle rowStyle = new RowStyle();
        ParagraphStyle paragraphStyle = new ParagraphStyle();
        paragraphStyle.setAlign(ParagraphAlignment.LEFT);
        CellStyle cellStyle = new CellStyle();
        cellStyle.setDefaultParagraphStyle(paragraphStyle);
        cellStyle.setVertAlign(XWPFTableCell.XWPFVertAlign.CENTER);
        rowStyle.setDefaultCellStyle(cellStyle);
        return rowStyle;
    }

    /**
     * 获取行样式
     *
     * @return
     */
    protected RowStyle getHeadRowStyle() {
        RowStyle rowStyle = new RowStyle();
        ParagraphStyle paragraphStyle = new ParagraphStyle();
        paragraphStyle.setAlign(ParagraphAlignment.CENTER);
        CellStyle cellStyle = new CellStyle();
        cellStyle.setDefaultParagraphStyle(paragraphStyle);
        cellStyle.setBackgroundColor("F2F2F2");
        rowStyle.setDefaultCellStyle(cellStyle);
        return rowStyle;
    }
}
